--CPML is the magic that makes the transforms work
local cpml = require "cpml"
local vertices = require "cube"

local mesh,shader

--initial position and angle of Camera
local cameraPos = cpml.vec3(0, 0, -15)
local angle     = cpml.vec2(0, 0)

--initial transforms for the cubes
local cube1 = cpml.mat4.identity()
cube1:translate(cube1, cpml.vec3(1,0,0))

local cube2 = cpml.mat4.identity()
cube2:translate(cube2, cpml.vec3(5,4,0))


--Custom vertex format add z to vertex position
local format = {
	{
		"VertexPosition",
		"float",
		3
	},
	{
		"VertexTexCoord",
		"float",
		2
	},
}

function love.load()
	--load texture 
	local image = love.graphics.newImage("box.png")

	love.graphics.setBackgroundColor(.5,.5,.5)

	--Creates mesh using our format and vertices. It will be drawn as a set of triangles
	--We set the texture to our image and load our shader.
	mesh = love.graphics.newMesh(format, vertices, "triangles")
	mesh:setTexture(image)
	shader = love.graphics.newShader("basicRender.glsl")

	--Sets DepthMode. Prevents items being drawn out of order
	love.graphics.setDepthMode("lequal", true)

end

function love.draw()


	love.graphics.push("all")
	--centers the world on the screen
	local w, h = love.graphics.getDimensions()
	love.graphics.translate(w/2, h/2)

	--Sends the view and model transforms to shader
	local view = matrix()
	shader:send("view", view)
	shader:send("model_matrix", makeProper(cube1))

	--Sets the shader
	love.graphics.setShader(shader)

	--Draw mesh and clear shader.
	love.graphics.setColor(1,0,0)
	love.graphics.draw(mesh)
	love.graphics.setShader()
	love.graphics.pop()


	--Repeat for cube 2
	love.graphics.push("all")
	--centers the world on the screen
	local w, h = love.graphics.getDimensions()
	love.graphics.translate(w/2, h/2)

	--Sends the view and model transforms to shader
	local view = matrix()
	shader:send("view", view)
	shader:send("model_matrix", makeProper(cube2))

	--Set the shader
	love.graphics.setShader(shader)

	--Draw mesh and clear shader.
	love.graphics.setColor(1,0,1)
	love.graphics.draw(mesh)
	love.graphics.setShader()
	love.graphics.pop()


	love.graphics.print("Angle: " .. angle.x .. ", " .. angle.y, 20, 20)
	love.graphics.print("Position: " .. cameraPos.x .. ", " .. cameraPos.y, 20, 20*2)
	love.graphics.print("Backspace to reset position and angle", 20, 20*3)
	love.graphics.print("Click and drag to rotate", 20, 20*4)
	love.graphics.print("WASD to move camera position", 20, 20*5)
	love.graphics.print("FPS " .. love.timer.getFPS(), 20, 20*6)
end

function love.update(dt)
	local speed = 5 * dt
	if love.keyboard.isDown("w", "up") then
		cameraPos.y = cameraPos.y - speed
	elseif love.keyboard.isDown("s", "down") then
		cameraPos.y = cameraPos.y + speed
	end
	if love.keyboard.isDown("a", "left") then
		cameraPos.x = cameraPos.x + speed
	elseif love.keyboard.isDown("d", "right") then
		cameraPos.x = cameraPos.x - speed
	end
end

--Function to merge camera transform and projection matrix(I think anyways)
function matrix()
	local mat = cpml.mat4.from_perspective(60, love.graphics.getWidth()/love.graphics.getHeight(), 0.1, 1000)
	local v = cpml.mat4()
	v:translate(v, cameraPos)
	v:rotate(v, -math.pi/2, cpml.vec3.unit_x)
	v:rotate(v, angle.y, cpml.vec3.unit_x)
	v:rotate(v, angle.x, cpml.vec3.unit_y)
	return makeProper(mat) * makeProper(v)
end


--Function to correct cpml incorrect matrix format.
function makeProper(mat)
	local m = {}
	for x = 1, 4 do
		local t = {}
		for y = 1, 4 do
			table.insert(t, mat[(y - 1) * 4 + x])
		end
		table.insert(m, t)
	end
	return cpml.mat4(m)
end

function love.keypressed(key)
	if key == "escape" then
		love.event.quit()
	elseif key == "backspace" then
		angle = cpml.vec2(0, 0)
		cameraPos = cpml.vec3(0, 0, -15)
	end
end

function love.mousepressed(x, y, button)
	if button == 1 then
		dragging = true
		love.mouse.setRelativeMode(true)
	end
end

function love.mousereleased(x, y, button)
	if button == 1 then
		dragging = false
		love.mouse.setRelativeMode(false)
	end
end

function love.mousemoved(x, y, dx, dy)
	if dragging then
		angle.x = angle.x + math.rad(dx * 0.5)
		angle.y = angle.y + math.rad(dy * 0.5)
	end
end

